#!/bin/sh -e

#
# Copyright (c) 2022 VMware, Inc. All Rights Reserved.
#

set -e -o pipefail

# Be receptive to core dumps
ulimit -c unlimited

# Allow high connection count per process (raise file descriptor limit)
ulimit -n 65536

# Init Java options - ORG_JAVA_OPTS will keep the original value when the script was started for the first time
JAVA_OPTS="${ORG_JAVA_OPTS:=${JAVA_OPTS:- }}"
export ORG_JAVA_OPTS

# Allow custom props to be inserted
test -f /custom-conf1.sh && . /custom-conf1.sh

# remove options not supported in OpenJDK 11
JAVA_OPTS=$( <<< "$JAVA_OPTS" sed -e 's/-XX:+UseParNewGC//' )

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/var/log/error_java_%p.log"
JAVA_OPTS+=" -Xlog:gc*:file=/var/log/gc.log:time,uptime,level,tags:filecount=10,filesize=10m"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null
then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

echo "Setting heap to ${JVM_HEAP:=1024m}"
if [ -z "$XMX" ] | [ -z "$XMS" ]; then
  echo "Setting XMS and XMX to JVM_HEAP: ${JVM_HEAP}"
  JAVA_OPTS+=" -Xms$JVM_HEAP -Xmx$JVM_HEAP"
else
  echo "Setting XMS to ${XMS} and XMX to ${XMX}"
  JAVA_OPTS+=" -Xms$XMS -Xmx$XMX"
fi


echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

HEAP_DUMP_PATH="/var/log/$( date '+%Y-%m-%d_%H:%M' )_heap_dump.hprof"

JAVA_OPTS+=" -Xss512K"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"
JAVA_OPTS+=" -XX:+UseStringDeduplication"


if [ -n "$DEBUG_PORT" ]; then
  echo "Enabling debugging on port: $DEBUG_PORT"
  JAVA_OPTS+=" -Xdebug -Xrunjdwp:transport=dt_socket,suspend=n,server=y,address=$DEBUG_PORT"
fi

if [ -n "$SERVER_PORT" ]; then
  JAVA_OPTS+=" -Dserver.port=$SERVER_PORT"
fi

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

# Tracked by https://jira.eng.vmware.com/browse/VRAE-51321
JAVA_OPTS+=" --add-opens=jdk.management/com.sun.management.internal=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/jdk.internal.misc=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/sun.nio.ch=ALL-UNNAMED  "
JAVA_OPTS+=" --add-opens=java.management/com.sun.jmx.mbeanserver=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=jdk.internal.jvmstat/sun.jvmstat.monitor=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/sun.reflect.generics.reflectiveObjects=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/java.io=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/java.nio=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/java.util=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/java.util.concurrent=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/java.util.concurrent.locks=ALL-UNNAMED "
JAVA_OPTS+=" --add-opens=java.base/java.lang=ALL-UNNAMED "


echo "setting enable.grpc.auth to true"
EXT_JAVA_OPTS+=" -Denable.grpc.auth=true -Denable.grpc.server.authz=true"

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

# Add ingress/load-balancert/proxy certificates in the java keystore if such are mounted,
# configure FIPS properties
#
source /usr/local/bin/setup_security.sh

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

#https://www.oracle.com/java/technologies/javase/jdk-11-relnote.html
#TLS 1.3 Half-Close Policy
JAVA_OPTS+=" -Djdk.tls.acknowledgeCloseNotify=true"


HUB_SM_CERT_PATH=/run/pinniped/certs

if [ -d  $HUB_SM_CERT_PATH ]
then
  read TRUST_STORE_PASSWORD < $JAVA_HOME/lib/security/cacerts.pass
  openssl pkcs12 -export -out /tmp/combined.pfx -inkey /run/pinniped/certs/tls.key -in /run/pinniped/certs/tls.crt -passout pass:
  openssl pkcs12 -in /tmp/combined.pfx -clcerts -nokeys -out /tmp/combined.cer -passin pass:
  keytool -noprompt -keystore "$JAVA_HOME/lib/security/cacerts" -alias pinniped -import -file "/tmp/combined.cer" -storepass $TRUST_STORE_PASSWORD -trustcacerts
  echo "Importing pinniped certs $JAVA_HOME/lib/security/cacerts"
  JAVA_OPTS+=" -Djavax.net.ssl.trustStore=$JAVA_HOME/lib/security/cacerts"
  JAVA_OPTS+=" -Djavax.net.ssl.trustStorePassword=$TRUST_STORE_PASSWORD"
fi

echo -e '\n\n==============\n\n\n'
status=0

##### Start the programs


cd "$TASK_MANAGER_HOME"

java \
    $JAVA_OPTS \
    -cp ${BCFIPS_JARS:+"$BCFIPS_JARS"}:.:${TASK_MANAGER_HOME}/libs/*  \
    com.vmware.ensemble.idem.manager.IdemTaskManagerApp --spring.config.additional-location="$APPLICATION_CONFIG_FILE"\
    "$@" 2>&1 &

PID=$!

wait $PID || status=$?
echo "jvm exited with status code $status"

exit "$status"
